enum BAUDRATE {
  //% block="4800"
  4800,
  //% block="1200"
  1200,
  //% block="2400"
  2400,
  //% block="9600"
  9600,
  //% block="19200"
  19200,
  //% block="38400"
  38400,
  //% block="57600"
  57600,
  //% block="76800"
  76800,
  //% block="115200"
  115200
}

enum AD {
  //% block="1-3:上下下(0x01)"
  0x01,
  //% block="1-3:下上下(0x02)"
  0x02,
  //% block="1-3:上上下(0x03)"
  0x03,
  //% block="1-3:下下上(0x04)"
  0x04,
  //% block="1-3:上下上(0x05)"
  0x05,
  //% block="1-3:下上上(0x06)"
  0x06,
  //% block="1-3:上上上(0x07)"
  0x07
}

enum LNSFAIoT_enum_parameter {
  //% block="温度值(℃)"
  LNSFAIoT_air_num_WenDu,
  //% block="湿度值(%)"
  LNSFAIoT_air_num_ShiDu,
  //% block="PM10含量(ug/m3)"
  LNSFAIoT_air_num_PM10,
  //% block="二氧化碳气体(CO2)含量(ppm)"
  LNSFAIoT_air_num_CO2,
  //% block="甲醛气体(CH2O)含量(ug/m3)"
  LNSFAIoT_air_num_CH2O,
  //% block="总挥发性有机化合物(TVOC)含量(ug/m3)"
  LNSFAIoT_air_num_TVOC,
  //% block="PM2.5含量(ug/m3)"
  LNSFAIoT_air_num_PM25
}

//% color="#3A5FFF" iconWidth=50 iconHeight=40
namespace RS485_air_quality_sensor {
  // % board="arduino,arduinonano,esp32,leonardo,mega2560,firebeetleesp32,firebeetleesp32e"
  //% block="初始化多合一空气质量传感器(RS485)设备地址(拨码开关)[AD] 接口[SER]RX:[RXD]TX:[TXD]波特率[BAUDRATE]" blockType="command"
  //% SER.shadow="dropdown" SER.options="SER"
  //% TXD.shadow="dropdown" TXD.options="TXD"
  //% RXD.shadow="dropdown" RXD.options="RXD"
  //% BAUDRATE.shadow="dropdown" BAUDRATE.options="BAUDRATE" BAUDRATE.defl="BAUDRATE.9600"
  //% AD.shadow="dropdown" AD.options="AD" AD.defl="AD.1-3:上下下"
  export function beginSerial(parameter: any, block: any) {
    let serial = parameter.SER.code;
    let tx = parameter.TXD.code;
    let rx = parameter.RXD.code;
    let baudrate = parameter.BAUDRATE.code;
    let ad = parameter.AD.code;
    let addr = 1;
    Generator.addInclude('LNSFAIoTs', '/*\n技术支持：关注微信公众号“人工智能素养教育共同体”\n开源gitee地址：https://gitee.com/LNSFAIoT/mindplus-rs485-all-in-one-air-quality-sensor\n测试硬件：\nhttps://shop221040643.taobao.com\n多合一空气质量传感器 图形化编程 自研 提供源码 中小学 创客 AI\nhttps://item.taobao.com/item.htm?id=678612499027\n注意：\n修改传感器的设备地址时，先接通电，在上电状态下通过拨码开关拨到想要的地址，\n再断电后重新上电即可用新设置的地址初始化。*/\n');
    // Generator.addInclude('BEIZHU', '');
    Generator.addInclude('LNSFAIoT_RS485_air_quality_sensors', '#include <LNSFAIoT_RS485_air_quality_sensor.h>');

    // Generator.addObject(`my2s`, `volatile float`, `LNSFAIoT_num_WenDu, LNSFAIoT_num_ShiDu; //解析传感器后的数据`);
    // Generator.addObject(`uints`, `unsigned int`, `LNSFAIoT_num_PM10, LNSFAIoT_num_CO2, LNSFAIoT_num_CH2O,LNSFAIoT_num_TVOC, LNSFAIoT_num_PM25;`);
    Generator.addObject(`LNSFAIoT_RS485_air_quality_sensorw`, `LNSFAIoT_RS485_air_quality_sensor`, `lnsfaiotair_rs485=LNSFAIoT_RS485_air_quality_sensor();`);

    if (ad == '0x01') {
      // Generator.addObject(`constint32_tS21`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x01,0x03,0x00,0x02,0x00,0x07,0xA5,0xC8};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=1;`);
      addr = 1;
    } else if (ad == '0x02') {
      // Generator.addObject(`constint32_tS22`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x02,0x03,0x00,0x02,0x00,0x07,0xA5,0xFB};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=2;`);
      addr = 2;

    } else if (ad == '0x03') {
      // Generator.addObject(`constint32_tS23`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x03,0x03,0x00,0x02,0x00,0x07,0xA4,0x2A};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=3;`);
      addr = 3;

    } else if (ad == '0x04') {
      // Generator.addObject(`constint32_tS2`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x04,0x03,0x00,0x02,0x00,0x07,0xA5,0x9D};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=4;`);
      addr = 4;

    } else if (ad == '0x05') {
      // Generator.addObject(`constint32_tS2`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x05,0x03,0x00,0x02,0x00,0x07,0xA4,0x4C};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=5;`);
      addr = 5;

    } else if (ad == '0x06') {
      // Generator.addObject(`constint32_tS2`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x06,0x03,0x00,0x02,0x00,0x07,0xA4,0x7F};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=6;`);
      addr = 6;

    } else if (ad == '0x07') {
      // Generator.addObject(`constint32_tS2`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x07,0x03,0x00,0x02,0x00,0x07,0xA5,0xAE};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=7;`);
      addr = 7;

    }


    // if (Generator.board === "arduino") {
    //   Generator.addSetup("SerialSetups", `${serial}.begin(${baudrate});`);
    // } else if (Generator.board === "arduinonano") {
    //   Generator.addSetup("SerialSetups", `${serial}.begin(${baudrate});`);
    // } else if (Generator.board === "firebeetleesp32") {
    //   Generator.addSetup("SerialSetups", `${serial}.begin(${baudrate});`);
    // } else if (Generator.board === "leonardo") {
    //   Generator.addSetup("SerialSetups", `${serial}.begin(${baudrate});`);
    // } else if (Generator.board === "mega2560") {
    //   Generator.addSetup("SerialSetups", `${serial}.begin(${baudrate});`);
    // } else if (Generator.board === "microbit") {
    //   Generator.addSetup(
    //     "SerialSetups",
    //     `${serial}.begin(${baudrate}, ${rx}, ${tx});`
    //   );
    // } else if (Generator.board === "esp32") {
    //   Generator.addSetup(
    //     "SerialSetups",
    //     `${serial}.begin(${baudrate}, ${rx}, ${tx});`
    //   );
    // }

    Generator.addSetup("SerialSetupS", `lnsfaiotair_rs485.begin(&${serial},${rx},${tx},${baudrate},${addr});`);


    //     Generator.addCode(`for(int i=0;i<8;i++){
    //             ${serial}.write(lnsfaiotair_rs485_wenxun[i]);
    //             }
    //             if ((${serial}.available())) {`);
    //     Generator.addCode(`
    //         if ((${serial}.read()==1) && (${serial}.read()==3) && (${serial}.read()==14)) {
    //           LNSFAIoT_list_serialdata.clear();
    //             while (!(!(${serial}.available()))) {
    //                 LNSFAIoT_list_serialdata.push_back((String(${serial}.read())));
    //                 yield();
    //             }
    //             LNSFAIoT_num_CO2 = (String(LNSFAIoT_list_serialdata[0]).toFloat()) * 256 + String(LNSFAIoT_list_serialdata[1]).toFloat() ;
    //             LNSFAIoT_num_CH2O = (String(LNSFAIoT_list_serialdata[2]).toInt()) * 256 + String(LNSFAIoT_list_serialdata[3]).toInt();
    //             LNSFAIoT_num_TVOC = (String(LNSFAIoT_list_serialdata[4]).toInt()) * 256 + String(LNSFAIoT_list_serialdata[5]).toInt();
    //             LNSFAIoT_num_PM25 = (String(LNSFAIoT_list_serialdata[6]).toFloat()) * 256 + String(LNSFAIoT_list_serialdata[7]).toFloat();
    //             LNSFAIoT_num_PM10 = (String(LNSFAIoT_list_serialdata[8]).toFloat()) * 256 + String(LNSFAIoT_list_serialdata[9]).toFloat() ;
    //             LNSFAIoT_num_WenDu = ((String(LNSFAIoT_list_serialdata[10]).toFloat()) * 256 + (String(LNSFAIoT_list_serialdata[11]).toFloat())) * 0.1;
    //             LNSFAIoT_num_ShiDu = ((String(LNSFAIoT_list_serialdata[12]).toFloat()) * 256 + (String(LNSFAIoT_list_serialdata[13]).toFloat())) * 0.1;

    //         }
    //     }
    // delay(1000);`);
  }


  //% board="arduino,arduinonano,microbit,leonardo,mega2560"
  //% block="初始化多合一空气质量传感器(RS485)设备地址(拨码开关)[AD] 接口[SSER]RX:[SSRXD]TX:[SSTXD]波特率[BAUDRATE]" blockType="command"
  //% SSER.shadow="dropdown" SSER.options="SSER"
  //% SSTXD.shadow="dropdown" SSTXD.options="SSTXD"
  //% SSRXD.shadow="dropdown" SSRXD.options="SSRXD"
  //% BAUDRATE.shadow="dropdown" BAUDRATE.options="BAUDRATE" BAUDRATE.defl="BAUDRATE.9600"
  //% AD.shadow="dropdown" AD.options="AD" AD.defl="AD.1-3:上下下"
  export function beginSoftSerial(parameter: any, block: any) {
    let tx = parameter.SSTXD.code;
    let rx = parameter.SSRXD.code;
    let sser = parameter.SSER.code;
    let baudrate = parameter.BAUDRATE.code;
    let ad = parameter.AD.code;
    let addr = 1;

    Generator.addInclude('LNSFAIoTs', '/*\n技术支持：关注微信公众号“人工智能素养教育共同体”\n开源gitee地址：https://gitee.com/LNSFAIoT/mindplus-rs485-all-in-one-air-quality-sensor\n测试硬件：\nhttps://shop221040643.taobao.com\n多合一空气质量传感器 图形化编程 自研 提供源码 中小学 创客 AI\nhttps://item.taobao.com/item.htm?id=678612499027\n注意：\n修改传感器的设备地址时，先接通电，在上电状态下通过拨码开关拨到想要的地址，\n再断电后重新上电即可用新设置的地址初始化。*/\n');
    Generator.addInclude('LNSFAIoT_RS485_air_quality_sensors', '#include <LNSFAIoT_RS485_air_quality_sensor.h>');

    Generator.addObject(`LNSFAIoT_RS485_air_quality_sensorwff`, `LNSFAIoT_RS485_air_quality_sensor`, `lnsfaiotair_rs485=LNSFAIoT_RS485_air_quality_sensor();`);

    // Generator.addObject(`my2sS`, `volatile float`, `LNSFAIoT_num_WenDu, LNSFAIoT_num_ShiDu; //解析传感器后的数据`);
    // Generator.addObject(`uintsS`, `unsigned int`, `LNSFAIoT_num_PM10, LNSFAIoT_num_CO2, LNSFAIoT_num_CH2O,LNSFAIoT_num_TVOC, LNSFAIoT_num_PM25;`);
    // Generator.addObject(`my3sS`, `SimpleList<String>`, ` LNSFAIoT_list_serialdata; `);

    if (ad == '0x01') {
      // Generator.addObject(`constint32_tS21`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x01,0x03,0x00,0x02,0x00,0x07,0xA5,0xC8};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=1;`);
      addr = 1;
    } else if (ad == '0x02') {
      // Generator.addObject(`constint32_tS22`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x02,0x03,0x00,0x02,0x00,0x07,0xA5,0xFB};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=2;`);
      addr = 2;

    } else if (ad == '0x03') {
      // Generator.addObject(`constint32_tS23`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x03,0x03,0x00,0x02,0x00,0x07,0xA4,0x2A};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=3;`);
      addr = 3;

    } else if (ad == '0x04') {
      // Generator.addObject(`constint32_tS2`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x04,0x03,0x00,0x02,0x00,0x07,0xA5,0x9D};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=4;`);
      addr = 4;

    } else if (ad == '0x05') {
      // Generator.addObject(`constint32_tS2`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x05,0x03,0x00,0x02,0x00,0x07,0xA4,0x4C};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=5;`);
      addr = 5;

    } else if (ad == '0x06') {
      // Generator.addObject(`constint32_tS2`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x06,0x03,0x00,0x02,0x00,0x07,0xA4,0x7F};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=6;`);
      addr = 6;

    } else if (ad == '0x07') {
      // Generator.addObject(`constint32_tS2`, `int32_t`, `lnsfaiotair_rs485_wenxun[8]={0x07,0x03,0x00,0x02,0x00,0x07,0xA5,0xAE};`);
      // Generator.addObject(`int`, `char`, `lnsf_addr=7;`);
      addr = 7;

    }

    Generator.addObject(
      "softSerialObjectSS",
      "SoftwareSerial",
      `lnsfaiotair485${sser}(${rx}, ${tx});`
    );

    Generator.addSetup("lnsfsoilSoftSerialSetup", `lnsfaiotair_rs485.begin(&lnsfaiotair485${sser},${baudrate},${addr});`);
    // Generator.addObject(
    //   "softSerialObjectss",
    //   "SoftwareSerial",
    //   `${sser}(${rx}, ${tx});`
    // );
    // Generator.addSetup("SoftSerialSetupss", `${sser}.begin(${baudrate});`);

    //     Generator.addCode(`for(int i=0;i<8;i++){
    //             ${sser}.write(lnsfaiotair_rs485_wenxun[i]);
    //             }
    //             if ((${sser}.available())) {`);
    //     Generator.addCode(`
    //         if ((${sser}.read()==1) && (${sser}.read()==3) && (${sser}.read()==14)) {
    //           LNSFAIoT_list_serialdata.clear();
    //             while (!(!(${sser}.available()))) {
    //                 LNSFAIoT_list_serialdata.push_back((String(${sser}.read())));
    //                 yield();
    //             }
    //             LNSFAIoT_num_CO2 = (String(LNSFAIoT_list_serialdata[0]).toFloat()) * 256 + String(LNSFAIoT_list_serialdata[1]).toFloat() ;
    //             LNSFAIoT_num_CH2O = (String(LNSFAIoT_list_serialdata[2]).toInt()) * 256 + String(LNSFAIoT_list_serialdata[3]).toInt();
    //             LNSFAIoT_num_TVOC = (String(LNSFAIoT_list_serialdata[4]).toInt()) * 256 + String(LNSFAIoT_list_serialdata[5]).toInt();
    //             LNSFAIoT_num_PM25 = (String(LNSFAIoT_list_serialdata[6]).toFloat()) * 256 + String(LNSFAIoT_list_serialdata[7]).toFloat();
    //             LNSFAIoT_num_PM10 = (String(LNSFAIoT_list_serialdata[8]).toFloat()) * 256 + String(LNSFAIoT_list_serialdata[9]).toFloat() ;
    //             LNSFAIoT_num_WenDu = ((String(LNSFAIoT_list_serialdata[10]).toFloat()) * 256 + (String(LNSFAIoT_list_serialdata[11]).toFloat())) * 0.1;
    //             LNSFAIoT_num_ShiDu = ((String(LNSFAIoT_list_serialdata[12]).toFloat()) * 256 + (String(LNSFAIoT_list_serialdata[13]).toFloat())) * 0.1;

    //         }
    //     }
    // delay(1000);`);

  }


  //% block="读取多合一空气质量传感器(RS485)采集的数据" blockType="command"
  export function serial_read() {
    Generator.addCode(`lnsfaiotair_rs485.LNSFAIoT_air_quality_sensor_RS485analysis();`);
  }

  // //% block="温度值(℃)" blockType="reporter"
  // //% values.shadow="reporter"
  // export function wendu() {
  //   Generator.addCode(`LNSFAIoT_num_WenDu`);
  // }
  // //% block="湿度值(%RH)" blockType="reporter"
  // //% values.shadow="reporter"
  // export function shidu() {
  //   Generator.addCode(`LNSFAIoT_num_ShiDu`);
  // }
  // //% block="PM10含量(ug/m3)" blockType="reporter"
  // //% values.shadow="reporter"
  // export function PM10() {
  //   Generator.addCode(`LNSFAIoT_num_PM10`);
  // }
  // //% block="二氧化碳气体(CO2)含量(ppm)" blockType="reporter"
  // //% values.shadow="reporter"
  // export function CO2() {
  //   Generator.addCode(`LNSFAIoT_num_CO2`);
  // }
  // //% block="甲醛气体(CH2O)含量(ug/m3)" blockType="reporter"
  // //% values.shadow="reporter"
  // export function CH2O() {
  //   Generator.addCode(`LNSFAIoT_num_CH2O`);
  // }
  // //% block="总挥发性有机化合物(TVOC)含量(ug/m3)" blockType="reporter"
  // //% values.shadow="reporter"
  // export function TVOC() {
  //   Generator.addCode(`LNSFAIoT_num_TVOC`);
  // }
  // //% block="PM2.5含量(ug/m3)" blockType="reporter"
  // //% values.shadow="reporter"
  // export function PM25() {
  //   Generator.addCode(`LNSFAIoT_num_PM25`);
  // }

  //% block=" (需先读取数据) 空气质量中[LNSFAIoT_enum_parameter]" blockType="reporter"
  //% LNSFAIoT_enum_parameter.shadow="dropdown" LNSFAIoT_enum_parameter.options="LNSFAIoT_enum_parameter" LNSFAIoT_enum_parameter.defl="LNSFAIoT_enum_parameter.LNSFAIoT_air_num_PH"
  export function LNSFAIoT_parameter(parameter: any, block: any) {
    let lnsfaiot_parameter = parameter.LNSFAIoT_enum_parameter.code;
    Generator.addCode(`lnsfaiotair_rs485.${lnsfaiot_parameter}_RS485func()`);
  }

  


  // //% block=" 用户标识 [user] 上报[json]" blockType="reporter"
  // //% user.shadow="string" user.defl=LNSF_AIoT
  // //% json.shadow="string"  json.defl=键值对
  // export function mqtt_post(parameter: any, block: any) {
  //   let json = parameter.json.code;
  //   let user = parameter.user.code;
  //   // Generator.addObject(`my6e`, `String`, `LNSFAIoT_ClientID;`);
  //   // Generator.addSetup(`my7e`, `LNSFAIoT_ClientID=${user};`);
  //   // Generator.addCode(`("{\\"id\\":\\""+LNSFAIoT_ClientID+"\\",\\"params\\":{\\""+${json}+"},\\"method\\":\\"thing.event.property.post\\"}").c_str()`);
  //   user = user.replace(/^\"|\"$/g, '');
  //   Generator.addCode(`("{\\"id\\":\\""+${user}+"\\",\\"params\\":{\\""+${json}+"},\\"method\\":\\"thing.event.property.post\\"}").c_str()`);
  // }

  // //% block=" json键值对:  [lable] 值 [values] " blockType="reporter"
  // //% lable.shadow="string" lable.defl=标识符
  // //% values.shadow="reporter"
  // //% values.shadow="number"  values.defl=1
  // export function lable_values(parameter: any, block: any) {
  //   let lable = parameter.lable.code;
  //   let values = parameter.values.code;

  //   Generator.addCode(`${lable}+"\\":"+${values}`);
  // }

  // //% block=" [val1] 和 [val2] " blockType="reporter"
  // //% val1.shadow="string" val1.defl=键值对
  // //% val1.shadow="string"  val2.defl=键值对
  // export function val_val(parameter: any, block: any) {
  //   let val1 = parameter.val1.code;
  //   let val2 = parameter.val2.code;
  //   Generator.addCode(`${val1}+",\\""+${val2}`);
  // }

  //% block="串口0打印显示多合一空气质量传感器(RS485)采集的数据" blockType="command"
  export function serial_println() {
    Generator.addSetup("SerialCC", `Serial.begin(9600);`);

    // Generator.addCode(`
    //       Serial.println("湿度: " + String(LNSFAIoT_num_ShiDu) + "%");
    //       Serial.println("温度: " + String(LNSFAIoT_num_WenDu) + "℃");
    //       Serial.println("PM10: " + String(LNSFAIoT_num_PM10) + "ug/m3");
    //       Serial.println("CO2: " + String(LNSFAIoT_num_CO2) + "ppm");
    //       Serial.println("CH2O: " + String(LNSFAIoT_num_CH2O) + "ug/m3");
    //       Serial.println("TVOC: " + String(LNSFAIoT_num_TVOC) + "ug/m3");
    //       Serial.println("PM2.5: " + String(LNSFAIoT_num_PM25) + "ug/m3");
    //       `);
    Generator.addCode(`lnsfaiotair_rs485.LNSFAIoT_air_quality_sensor_RS485println();`);


  }

} 
